//+------------------------------------------------------------------+
//|                                                  a-TLE_v.1.6.mqh |
//|                                             . aka KimIV |
//|                                              http://www.kimiv.ru |
//|                                                                  |
//| 20.09.2005    (Three Level Exit).     |
//|               .                    |
//|                 .                   |
//| 22.09.2005 v.1.0            |
//|             .                                    |
//| 22.09.2005 v.1.1       .       |
//| 22.09.2005 v.1.2      .         |
//| 22.09.2005 v.1.3      .     |
//| 22.09.2005 v.1.4      .       |
//| 23.09.2005 v.1.5     .       |
//| 14.10.2005 v.1.6     .                     |
//| 23.11.2005 v.1.6      .       |
//|                                                                  |
//|         start               |
//|   if (UseTrailing) TrailingPositions();                          |
//+------------------------------------------------------------------+

//-------    -----------------------------------
extern string _Parameters_Trailing = "-----  ";
extern bool UseTrailing  = True;   //  
extern int  LevelProfit1 = 20;     //   
extern int  LevelMoving1 = 1;      //   
extern int  LevelProfit2 = 35;     //   
extern int  LevelMoving2 = 20;     //   
extern int  LevelProfit3 = 55;     //   
extern int  LevelMoving3 = 30;     //   
extern int  TrailingStop = 40;     //  
extern int  TrailingStep = 5;      //  


//+------------------------------------------------------------------+
//|                                              |
//+------------------------------------------------------------------+
void TrailingPositions() {
  for (int i=0; i<OrdersTotal(); i++) {
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      if (OrderSymbol()==Symbol() && OrderMagicNumber()==MAGIC) {
        ThreeLevelSystemOfOutput();
      }
    }
  }
}

//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
void ThreeLevelSystemOfOutput() {
  int pr=ProfitPosition(), sl=StopLossInPoint();
  int sp=MarketInfo(OrderSymbol(), MODE_SPREAD);

  if (pr>LevelProfit1 && pr<=LevelProfit2 && sl<LevelMoving1) ModifyStopLossInPoint(LevelMoving1);
  if (pr>LevelProfit2 && pr<=LevelProfit3 && sl<LevelMoving2) ModifyStopLossInPoint(LevelMoving2);
  if (pr>LevelProfit3 && sl<LevelMoving3) ModifyStopLossInPoint(LevelMoving3);
  if (pr>LevelMoving3+TrailingStop+TrailingStep) {
    SimpleTrailingPositions();
  }
}

//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
void SimpleTrailingPositions() {
  double pBid, pAsk, pp=MarketInfo(OrderSymbol(), MODE_POINT);

  if (OrderType()==OP_BUY) {
    pBid=MarketInfo(OrderSymbol(), MODE_BID);
    if (pBid-OrderOpenPrice()>TrailingStop*pp) {
      if (OrderStopLoss()<pBid-(TrailingStop+TrailingStep-1)*pp) {
        ModifyStopLoss(pBid-TrailingStop*pp, clModifyBuy);
        return;
      }
    }
  }
  if (OrderType()==OP_SELL) {
    pAsk=MarketInfo(OrderSymbol(), MODE_ASK);
    if (OrderOpenPrice()-pAsk>TrailingStop*pp) {
      if (OrderStopLoss()>pAsk+(TrailingStop+TrailingStep-1)*pp || OrderStopLoss()==0) {
        ModifyStopLoss(pAsk+TrailingStop*pp, clModifySell);
        return;
      }
    }
  }
}

//+------------------------------------------------------------------+
//|   StopLoss                                          |
//| :                                                       |
//|   ldStopLoss -  StopLoss                                  |
//+------------------------------------------------------------------+
void ModifyStopLoss(double ldStopLoss, color clModify) {
  bool fm;         //   
  int  err;

  if (IsTradeAllowed()) {
    fm=OrderModify(OrderTicket(),OrderOpenPrice(),ldStopLoss,OrderTakeProfit(),0,clModify);
    if (!fm) {
      err=GetLastError();
      Print("Error(",err,") modifying: ",ErrorDescription(err));
    }
    if (fm && UseSound) PlaySound(NameFileSound);
  }
}

//+------------------------------------------------------------------+
//|   StopLoss                                          |
//| :                                                       |
//|   ldStopLoss -  StopLoss                                  |
//+------------------------------------------------------------------+
void ModifyStopLossInPoint(int pp) {
  bool   fm;
  color  clModify;
  double ldSL=0, mp=MarketInfo(OrderSymbol(), MODE_POINT);
  int    err;

  if (OrderType()==OP_BUY) {
    ldSL=OrderOpenPrice()+pp*mp;
    clModify=clModifyBuy;
  }
  if (OrderType()==OP_SELL) {
    ldSL=OrderOpenPrice()-pp*mp;
    clModify=clModifySell;
  }
  if (IsTradeAllowed()) {
    fm=OrderModify(OrderTicket(),OrderOpenPrice(),ldSL,OrderTakeProfit(),0,clModify);
    if (fm) {
      if (UseSound) PlaySound(NameFileSound);
    } else {
      err=GetLastError();
      Print("Error(",err,") modifying: ",ErrorDescription(err));
    }
  }
}

//+------------------------------------------------------------------+
//|                                   |
//+------------------------------------------------------------------+
int ProfitPosition() {
  double pBid, pAsk, pp=MarketInfo(OrderSymbol(), MODE_POINT);
  double pr=0;

  if (OrderType()==OP_BUY) {
    pBid=MarketInfo(OrderSymbol(), MODE_BID);
    pr=(pBid-OrderOpenPrice())/pp;
  }
  if (OrderType()==OP_SELL) {
    pAsk=MarketInfo(OrderSymbol(), MODE_ASK);
    pr=(OrderOpenPrice()-pAsk)/pp;
  }
  return(MathRound(pr));
}

//+------------------------------------------------------------------+
//|  StopLoss                               |
//+------------------------------------------------------------------+
int StopLossInPoint() {
  double pp=MarketInfo(OrderSymbol(), MODE_POINT);
  double sp=0;

  if (OrderType()==OP_BUY) {
    sp=(OrderStopLoss()-OrderOpenPrice())/pp;
  }
  if (OrderType()==OP_SELL) {
    sp=(OrderOpenPrice()-OrderStopLoss())/pp;
  }
  if (OrderStopLoss()==0) sp=-OrderOpenPrice()/pp;
  return(MathRound(sp));
}
//+------------------------------------------------------------------+

